<?php

namespace Drupal\lightbox_media\Plugin\Field\FieldFormatter;

use Drupal\Component\Utility\Html;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;

/**
 * Trait for the 'lightbox_media_container' formatter.
 */
trait LightboxMediaContainerFormatterTrait {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'caption_field' => '',
    ] + parent::defaultSettings();
  }

  /**
   * List all the text fields of the current entity.
   *
   * @return array
   *   The entity text fields.
   */
  protected function getEntityFieldsByType(array $types): array {

    $found_fields = [];

    // Get the entity type and bundle from the field definition.
    $entity_type_id = $this->fieldDefinition->getTargetEntityTypeId();
    $bundle = $this->fieldDefinition->getTargetBundle();

    // Get all fields for this entity type and bundle.
    $fields = $this->entityFieldManager->getFieldDefinitions($entity_type_id, $bundle);

    foreach ($fields as $field_name => $field_definition) {
      // Skip base fields if needed.
      if ($field_definition->getFieldStorageDefinition()->isBaseField()) {
        continue;
      }
      // Check if the field type is in the list of wanted types.
      if (in_array($field_definition->getType(), $types)) {
        $found_fields[$field_name] =
          $field_definition->getLabel() . ' (' . $field_name . ')';
      }
    }

    return $found_fields;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {

    $elements = parent::settingsForm($form, $form_state);

    $available_fields_for_caption =
      $this->getEntityFieldsByType(['string', 'string_long']);
    $elements['caption_field'] = [
      '#title' => $this->t('Caption field (aria-label)'),
      '#type' => 'select',
      '#default_value' => $this->getSetting('caption_field'),
      '#empty_option' => $this->t('None'),
      '#options' => $available_fields_for_caption,
      '#description' => $this->t('Specify the field of the current entity that we should use as a title or aria-label for the lightbox dialog.'),
    ];

    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {

    $summary = parent::settingsSummary();

    $settings = $this->getSettings();

    if (!empty($settings['caption_field'])) {
      $summary[] = $this->t('Caption field: @field', ['@field' => $settings['caption_field']]);
    }

    return $summary;
  }

  /**
   * {@inheritDoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {

    $elements = parent::viewElements($items, $langcode);

    $entity = $items->getEntity();

    $container_id = $this->buildGroupId($items);

    $prefix = '<div data-lightbox-container="' . $container_id . '"';

    // Fetch the settings for the formatter.
    $formatter_settings = $this->getSettings();
    $caption_field = $formatter_settings['caption_field'] ?? NULL;
    if (!empty($caption_field) && $entity->hasField($caption_field)) {
      $caption_field = $entity->get($caption_field);
      if (!$caption_field->isEmpty()) {
        $caption = $caption_field->value;
      }
    }

    if (!empty($caption)) {
      $prefix .= ' data-lightbox-title="' . Html::escape($caption) . '"';
    }

    $prefix .= '>';

    $elements['#prefix'] = $prefix;
    $elements['#suffix'] = '</div>';
    return $elements;
  }

  /**
   * Build the group id from the entity and the current field.
   */
  protected function buildGroupId(FieldItemListInterface $items) {
    $entity = $items->getEntity();
    return $entity->getEntityTypeId() . '-' . $entity->bundle() . '-' . $entity->id() . '-' . $items->getName();
  }

}
